@@ -1,4 +1,4 @@
-# $Id: P40.pm 109 2006-04-07 20:15:31Z fil $
+# $Id: P40.pm 198 2007-02-28 18:45:18Z fil $
 package t::P40;
 use strict;
 
@@ -42,7 +42,3 @@ sub twothing
 1;
 __END__
 
-$Log$
-Revision 1.1  2006/04/07 20:15:31  fil
-Initial revision
-
